#ifndef _Builtin_
#define _Builtin_

#include <stdio.h>
#include <string.h>

/**
 * Die Klasse Builtin verwaltet die internen Kommandos eines Interpreters.
 *
 * @author Arbeitsgruppe EOS
 */
class Builtin {
private:
    /**
     * Definiert einen Methodenaufruf fr ein Kommando.
     *
     * @param CmdFunction Name der Methode die als Kommando ausgefhrt werden soll
     * @param argc Anzahl der bergebenen Parameter
     * @param argv Liste der bergebenen Parameter, wobei Parameter 0 
     *             dem Namen des aufgerufenen Kommandos entspricht
     * @return Liefert den Ausfhrungsstatus des Kommandos (true entspricht erfolgreich beendet).
     */
    typedef int (Builtin::*CmdFunction)(int argc, char **argv) const;
    
    /**
     * Definiert einen Struktur, die einen Kommandostring auf eine Methode abbildet.
     */
    struct name {
        const char *cmd;
        Builtin::CmdFunction func;
    };
    
    /**
     * Liste der verwalteten internen Kommandos.
     */
    static const name table[];
    
    /**
     * Pointer auf den aufrufenden Interpreter.
     */
    class BSShell *_parent;
    
    /**
     * Das Kommando beendet den Interpreter.
     */
    int exit(int, char **) const;
    
    /**
     * Das Kommando gibt eine Liste der Kommandos des Interpreters aus.
     */
    int help(int, char **) const;
    
    /**
     * Das Kommando beendet den Interpreter.
     */
    int memory(int, char **) const;
    
    /**
     * Das Kommando gibt eine Liste der Kommandos des Interpreters aus.
     */
    int memtest(int, char **) const;
public:    
    /**
     * Erzeugt eine Instanze der Klasse Builtin. 
     */
    Builtin();
    ~Builtin();
  
    /**
     * Fhrt ein Kommandos des Interpreters anhand der bergebenen 
     * Referenznummer und Parameter aus.
     * Es werden nur interne Kommandos ausgefhrt, die Ausfhrun von Kommandos, 
     * die nicht im Interpreter implementiert sind, ist nicht vorgesehen.
     *
     * @param refnr Referenznummer des auszufhrenden Kommandos
     * @param argc Anzahl der bergebenen Parameter
     * @param argv Liste der bergebenen Parameter, wobei Parameter 0 
     *             dem Namen des aufgerufenen Kommandos entspricht
     * @return Liefert den Ausfhrungsstatus des Kommandos.
     */
    int execute(int refnr, int argc, char **argv) const;
    
    /**
     * Testet ob der bergebene Kommandostring einem Kommandoaufruf entspricht.
     *
     * @param cmdname Kommandostring der getestet werden soll
     * @return Liefert die Referenznummer des Kommandos oder -1 wenn ein 
     *         Fehler aufgetreten ist.
     */
    int is_builtin(const char *cmdname) const; 
    
    /**
     * Testet ob der bergebene Kommandostring einem Kommandoaufruf entspricht.
     *
     * @param cmdname Kommandostring der getestet werden soll
     * @return Liefert die Referenznummer des Kommandos oder -1 wenn ein 
     *         Fehler aufgetreten ist.
     */
    void setParent(BSShell *parent); 
};

#endif //_Builtin_
