#ifndef __fsfile_include__
#define __fsfile_include__

#include <sys/types.h>
#include "FSObject.hh"

class FSFile : public FSObject {
protected:
    /** current position of the read/write counter */
    size_t _filepos;
    /** current size of the file */
    size_t _filesize;
public:
    FSFile(FSLogic *logic) : FSObject(logic) {}
    FSFile(FSLogic *logic, const char* name) : FSObject(logic, name) {}
    
    /**
     * Sets the position of the read/write pointer of the open file .
     *
     * SEEK_SET     The offset is set to offset bytes.
     * SEEK_CUR     The offset is set to its current location plus offset bytes.
     * SEEK_END     The offset is set to the size of the file plus offset bytes.
     *
     * @param offset offset of new position
     * @param whence startpoint to count from (default is SEEK_CUR)
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int lseek(size_t offset, int whence = 0);
    
    /**
     * Reads up to count bytes from file into the buffer starting at buffer.
     *
     * @param buffer pointer to readbuffer
     * @param count number of bytes to be read
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int read(caddr_t buffer, size_t size);

    /**
     * Writes up to count bytes to the file from the buffer starting at buffer.
     *
     * @param buffer pointer to readbuffer
     * @param count number of bytes to be read
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int write(const caddr_t buffer, size_t size);

    /**
     * Identifies this object as a file.
     */
    Type type() { return Directory; }
};

#endif // __fsfile_include__
