#ifndef __FSLogic_include__
#define __FSLogic_include__

#include "FSDevice.hh"

class FSDirectory;
class FSFile;

class FSLogic : public FSDevice {
public:
    FSLogic(const char* name) : FSDevice(name) {}
    ~FSLogic() {}

    /**
     * Creates a new directory with the specified path.
     *
     * @param path absolut pathname of the new directory
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int mkdir(char *path);
    
    /**
     * Deletes the director with the specified pathname.
     *
     * @param path abolut pathname of the directory
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int rmdir(char *path);
    
    /**
     * Opens the specified directory. A directory is closed by deleting the 
     * coresponding directory object.
     *
     * @param apath absolut pathname of the directory
     * @param directory pointer to the directory object
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int opendir(char *apath, FSDirectory *directory);
    
    /**
     * Opens the specified file using read/write mode. A file is closed by 
     * deleting the coresponding file object.
     *
     * @param apath absolut pathname of the file
     * @param flags mode used for this file
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int openfile(char *apath, int flags, FSFile *file);

    /**
     * Reads the data of a file/directory in a specified buffer.
     *
     * @param buffer buffer to write to
     * @param cluster first cluster/block of file/directory in the filesystem
     * @param position position in file/directory to start reading
     * @param size number of bytes to be read
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int read(caddr_t buffer, size_t cluster, size_t position, size_t size);
    
    /**
     * Writes the data from a specified buffer in a file/directory.
     *
     * @param buffer buffer to read from
     * @param cluster first cluster/block of file/directory in the filesystem
     * @param position position in file/directory to start writing
     * @param size number of bytes to be write
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int write(const caddr_t buffer, size_t cluster, size_t position, size_t size);
};
      
#endif // __FSLogic_include__
