#ifndef __FSObject_include__
#define __FSObject_include__

#include "FSResult.hh"
#include "FSLogic.hh"

class FSDirectory;
class FSFile;

class FSObject {
protected:
    /** name of the object, provided by the filesystem client */
    const char* _name;
    /** stores the device that implements the file system logic */
    FSLogic* _logic;
    /** current size in the filesystem */
    size_t _size;
    /**  first cluster/block in the filesystem */
    size_t _start;
public:
    typedef enum { Directory, File, Unknown } Type;

    FSObject(FSLogic *logic) : _name(0), _logic(logic) {}
    FSObject(FSLogic *logic, const char* name) : _name(name), _logic(logic) {}
    virtual ~FSObject() {}

    // A public type of filesystem objects
    virtual Type type () { return Unknown; }

    /**
     * Changes the name in the filesystem.
     *
     * @param name new name of the file/directory
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int rename(const char* name);

    /**
     * Removes the object from the filessystem.
     *
     * @return Returns 0 if successfule otherwise the status as negative number.
     */
    int remove();
};
      
#endif // __FSObject_include__
