#ifndef __Block_Device_h__
#define __Block_Device_h__

#include <sys/types.h>
#include "BlockDeviceError.hh"

/**
 * This class defines the standard interface for block oriented devices.
 * It contains standard data types and methods for synchronous i/o.
 */
class BlockDevice {
public:
    typedef unsigned long BlockNo;
    typedef unsigned long BlockCount;
    typedef unsigned long BlockSize;

    /**
     * Selects the preferred logical block size.
     *
     * @param count number of blocks this device can use
     * @param size  logical size of a block (Byte)
     */
    virtual BlockDeviceError setup(BlockCount count, BlockSize size) = 0;

    /**
     * Gets characteristics of device.
     *
     * @return Returns the size of block of the device.
     */
    virtual BlockSize blocksize() = 0;

    /**
     * Gets characteristics of device.
     *
     * @return Returns the number of block this device can use.
     */
    virtual BlockCount count() = 0;

    /**
     * Reads multiple blocks at once from the device.
     *
     * @param block number of startblock
     * @param ptr   pointer to readbuffer
     * @param count number of blocks to read
     * @return Returns an appropriate error object (BlockDeviceError) for the operation.
     */
    virtual BlockDeviceError read(BlockNo block, caddr_t ptr, BlockCount count) = 0;

    /**
     * Writes multiple blocks at once on the device.
     *
     * @param block number of startblock
     * @param ptr   pointer to writebuffer
     * @param count number of blocks to read
     * @return Returns an appropriate error object (BlockDeviceError) for the operation.
     */
    virtual BlockDeviceError write(BlockNo block, caddr_t ptr, BlockCount count) = 0;
    virtual ~BlockDevice() {}
};

#endif /* __Block_Device_h__ */
