#ifndef __FS_Device_h__
#define __FS_Device_h__

#include <fstream>
#include <iostream>
#include <stdlib.h>
#include "BlockDevice.hh"

#define BLOCKDEVSIZE 1024

using namespace std;

/**
 * This class implements the standard interface for block oriented devices. 
 */
class FSDevice : public virtual BlockDevice {
private:
    /** stores the allowed access type */
    enum AccessType { RW, R, NONE } access;
protected:
    /** stores the number of blocks */
    unsigned long _blockcnt;
    /** stores the size of a block */
    unsigned long _blocksize;    
    /** stores the stream used for this device */
    fstream _stream;
public:
    FSDevice(const char* name);
    ~FSDevice();
    
    /**
     * Selects the preferred logical block size.
     *
     * @param count number of blocks this device can use
     * @param size  logical size of a block (Byte)
     */
    BlockDeviceError setup(BlockCount count, BlockSize size);

    /**
     * Gets characteristics of device.
     *
     * @return Returns the size of the usable space on the device.
     */
    BlockSize size();
      
    /**
     * Gets characteristics of device.
     *
     * @return Returns the size of block of the device.
     */
    BlockSize blocksize();

    /**
     * Gets characteristics of device.
     *
     * @return Returns the number of block this device can use.
     */
    BlockCount count();

    /**
     * Reads multiple blocks at once from the device.
     *
     * @param block number of startblock
     * @param ptr   pointer to readbuffer
     * @param count number of blocks to read
     * @return Returns an appropriate error object (BlockDeviceError) for the operation.
     */
    BlockDeviceError read(BlockNo block, caddr_t ptr, BlockCount count);

    /**
     * Writes multiple blocks at once on the device.
     *
     * @param block number of startblock
     * @param ptr   pointer to writebuffer
     * @param count number of blocks to read
     * @return Returns an appropriate error object (BlockDeviceError) for the operation.
     */
    BlockDeviceError write(BlockNo block, caddr_t ptr, BlockCount count);
};

#endif /* __FS_Device_h__ */
