#ifndef _BitMask_interface_
#define _BitMask_interface_

#include "MemoryStrategy.hh"

/**
 *
 * Bitmask ist eine Strategie, die davon ausgeht, dass immer nur 
 * Speicherbloecke angefordert werden die der Pagesize entsprechen.
 * Daher verwalltet es ein Bitfeld, in dem vermerkt wird ob eine Page benutzt
 * ist ( Bit gesetzt 1) oder nicht ( Bit nicht gesetzt 0)
 * Die manage Member-Funktion bekommt ein Stueck Speicher, welches 
 * sie wie folgt einteilt
 *
 * _bitmaskadr      _memadr
 * ________________________________________________________________________
 * | Bitmaske      |    Pages                                             |
 * |_______________|____|____|____________________________________________|
 * { _bitmasksize  }    {_pagesize (variable Zweierpotenz)}
 * es muss immer noch eine Spezialisierung vorgenommen werden damit dir Pagegroesse bekannt ist
 */
class BitMask : public MemoryStrategy {
protected:
    /**
     * Gre einer Seite in Bit
     */
    size_t _pagesize;
    
    /**
     * Adresse ab der Speicherblcke belegt werden knnen
     */
    size_t* _bitmaskadr;
    
    /**
     * Anzahl der Bits in der Bitmaske, entspricht der Anzahl der Pages
     */
    size_t  _bitmasksize;
        
    void* get(size_t size = 1);
    void release(caddr_t memadr);
public:
    BitMask() {}
    size_t GetPageSize() { return 1 << _pagesize;}
    size_t GetSizeBitMask() { return _bitmasksize;}
    
    void showBlocks();
    void manage(caddr_t memadr, size_t memsize, size_t alignment = 2);
};

#endif // _BitMask_interface_
