#include "FileSystem.hh"

FileSystem::FileSystem(FSLogic* logic) : _fserrno(0), _logic(logic) {
    for(unsigned int slot = 0; slot < MAXFD; slot++) 
        _descriptors[slot] = NULL;
}

FileSystem::~FileSystem() {
    FSFile* file;

    for(unsigned int slot = 0; slot < MAXFD; slot++) {
        file = _descriptors[slot];
        if (file) {
            delete file;
            _descriptors[slot] = NULL;
        }
    }
}

char* FileSystem::absolutPath(const char *path) {
    char* result;
    int length = strlen(path);
    
    if (result[0] == '/') {
        result = new char[length + 1];
        strcpy(result, path);
    } else {
        char* wd = _current->pwd();        
        result = new char[length + strlen(wd) + 1];        
        strcpy(result,wd);
        strcpy(result + length, path);
        delete(wd);
    }
    
    if (length != 1 && result[length -1] == '/') result[length -1] = '\0';
    return result;
}

void FileSystem::pwd() {
    fprintf(stdout, "%s", _current->pwd());
}

int FileSystem::chdir(const char *path) {
    char* apath = absolutPath(path);
    FSDirectory* directory = new FSDirectory(_logic);

    int result = _logic->opendir(apath, directory);
    delete(apath);
    if (result < 0) {
        _fserrno = (-1 * result);
        return -1;
    } else {
        delete _current;
        _current = directory;
    }
    return 0;
}

int FileSystem::mkdir(const char *path) {
    char* apath = absolutPath(path);
    int result = _logic->mkdir(apath);
    delete(apath);
    if (result < 0) {
        _fserrno = (-1 * result);
        return -1;
    }
    return 0;
}

int FileSystem::rmdir(const char *path) {
    char* apath = absolutPath(path);

    int result = _logic->rmdir(absolutPath(path));
    delete(apath);
    if (result < 0) {
        _fserrno = (-1 * result);
        return -1;
    }
    return 0;
}

int FileSystem::readdir() {
    int result = _current->readdir();
    if (result < 0) {
        _fserrno = (-1 * result);
        return -1;
    }
    return 0;
}

int FileSystem::open(const char *path, int flags) {
    char* apath = absolutPath(path);
    FSFile* file = new FSFile(_logic);
    
    int result = _logic->openfile(apath, flags, file);
    delete(apath);
    if (result >= 0) {
        for(unsigned int slot = 0; slot < MAXFD; slot++) {
            if (!_descriptors[slot]) {
                _descriptors[slot] = file;
                return slot;
            }
        }
    } 
    return -1;
} 

int FileSystem::close(int fd) {
    FSFile* file = _descriptors[fd];
    if (file) {
        delete file;
        _descriptors[fd] = NULL;
    }
    return 0;
}

int FileSystem::lseek(int fd, size_t offset, int whence) {
    int result = ((FSFile*)_descriptors[fd])->lseek(offset, whence);
    if (result < 0) {
        _fserrno = (-1 * result);
        return -1;
    }
    return result;
}

int FileSystem::read(int fd, caddr_t buffer, size_t count) {
    int result = ((FSFile*)_descriptors[fd])->read(buffer, count);
    if (result < 0) {
        _fserrno = (-1 * result);
        return -1;
    }
    return result;
}

int FileSystem::write(int fd, const caddr_t buffer, size_t count) {    
    int result = ((FSFile*)_descriptors[fd])->write(buffer, count);
    if (result < 0) {
        _fserrno = (-1 * result);
        return -1;
    }
    return result;
}
