#include "FSDevice.hh"

FSDevice::FSDevice(const char* name) : _blockcnt(1) {
    _stream.open(name);
    if (_stream.is_open()) { 
        // get length of file:
        _stream.seekg (0, ios::end);
        _blocksize = _stream.tellg();
        _stream.seekg (0, ios::beg);

        // set standard blocksize
        if (BLOCKDEVSIZE < _blocksize)
            setup(_blocksize / BLOCKDEVSIZE, BLOCKDEVSIZE);
    } else {
        fprintf(stderr, "Error: Device could not be opened!\n");
    }
}

FSDevice::~FSDevice () {
    _stream.close();
}

BlockDeviceError FSDevice::setup(BlockCount count, BlockSize size) {
    const char* msg = (const char*)0;
    
    _blocksize = size;
    _blockcnt  = count;

    return BlockDeviceError(msg);
}

BlockDevice::BlockSize FSDevice::size() {
    return (BlockDevice::BlockSize)(_blocksize * _blockcnt);
}

BlockDevice::BlockSize FSDevice::blocksize() {
    return (BlockDevice::BlockSize)_blocksize;
}

BlockDevice::BlockCount FSDevice::count() {
    return (BlockDevice::BlockCount)_blockcnt;
}

BlockDeviceError FSDevice::read(BlockNo block, caddr_t ptr, BlockCount count) {
    const char* msg = (const char*)0;
    
    if (block < _blockcnt) {
        unsigned long streampos = block * _blocksize;
        if ((block + count) <= _blockcnt) {
            unsigned long length = streampos + (count * _blocksize);
    
            _stream.seekg(streampos, ios::beg);
            _stream.read((char*)ptr, length);
        } else {
            msg = "Error: trying to read to many blocks!\n";
        }
    } else {
        msg = "Error: startblock is out of range!\n";
    }
    return BlockDeviceError(msg);
}

BlockDeviceError FSDevice::write(BlockNo block, caddr_t ptr, BlockCount count) {
    const char* msg = (const char*)0;
    
    if (block < _blockcnt) {
        unsigned long streampos = block * _blocksize;
        if ((block + count) <= _blockcnt) {
            unsigned long length = streampos + (count * _blocksize);
    
            _stream.seekg(streampos, ios::beg); //ToFix: ios unbekannt
            _stream.write((char*)ptr, length);
        } else {
            msg = "Error: trying to write to many blocks!\n";
        }
    } else {
        msg = "Error: startblock is out of range!\n";
    }
    return BlockDeviceError (msg);
}
